@@ -7,21 +7,23 @@ import (
 	"errors"
 	"fmt"
 	"io"
+	"maps"
 	"net"
 	"os"
+	"slices"
 	"strings"
 	"time"
 
-	"github.com/containers/common/libnetwork/pasta"
-	"github.com/containers/common/libnetwork/types"
-	"github.com/containers/common/pkg/config"
-	"github.com/containers/common/pkg/secrets"
-	"github.com/containers/image/v5/manifest"
 	"github.com/containers/podman/v5/libpod/define"
 	"github.com/containers/podman/v5/libpod/lock"
-	"github.com/containers/storage"
 	spec "github.com/opencontainers/runtime-spec/specs-go"
 	"github.com/sirupsen/logrus"
+	"go.podman.io/common/libnetwork/pasta"
+	"go.podman.io/common/libnetwork/types"
+	"go.podman.io/common/pkg/config"
+	"go.podman.io/common/pkg/secrets"
+	"go.podman.io/image/v5/manifest"
+	"go.podman.io/storage"
 	"golang.org/x/sys/unix"
 )
 
@@ -145,9 +147,9 @@ type ContainerState struct {
 	// by containers/storage.
 	Mountpoint string `json:"mountPoint,omitempty"`
 	// StartedTime is the time the container was started
-	StartedTime time.Time `json:"startedTime,omitempty"`
+	StartedTime time.Time `json:"startedTime"`
 	// FinishedTime is the time the container finished executing
-	FinishedTime time.Time `json:"finishedTime,omitempty"`
+	FinishedTime time.Time `json:"finishedTime"`
 	// ExitCode is the exit code returned when the container stopped
 	ExitCode int32 `json:"exitCode,omitempty"`
 	// Exited is whether the container has exited
@@ -188,6 +190,7 @@ type ContainerState struct {
 	BindMounts map[string]string `json:"bindMounts,omitempty"`
 	// StoppedByUser indicates whether the container was stopped by an
 	// explicit call to the Stop() API.
+	// Warning: This field does persist across system reboots.
 	StoppedByUser bool `json:"stoppedByUser,omitempty"`
 	// RestartPolicyMatch indicates whether the conditions for restart
 	// policy have been met.
@@ -229,8 +232,8 @@ type ContainerState struct {
 
 	// Following checkpoint/restore related information is displayed
 	// if the container has been checkpointed or restored.
-	CheckpointedTime time.Time `json:"checkpointedTime,omitempty"`
-	RestoredTime     time.Time `json:"restoredTime,omitempty"`
+	CheckpointedTime time.Time `json:"checkpointedTime"`
+	RestoredTime     time.Time `json:"restoredTime"`
 	CheckpointLog    string    `json:"checkpointLog,omitempty"`
 	CheckpointPath   string    `json:"checkpointPath,omitempty"`
 	RestoreLog       string    `json:"restoreLog,omitempty"`
@@ -300,6 +303,11 @@ type ContainerArtifactVolume struct {
 	// the title annotation exist.
 	// Optional. Conflicts with Title.
 	Digest string `json:"digest"`
+	// Name is the name that should be used for the path inside the container. When a single blob
+	// is mounted the name is used as is. If multiple blobs are mounted then mount them as
+	// "<name>-x" where x is a 0 indexed integer based on the layer order.
+	// Optional.
+	Name string `json:"name,omitempty"`
 }
 
 // ContainerSecret is a secret that is mounted in a container
@@ -534,12 +542,7 @@ func (c *Container) Dependencies() []string {
 		return []string{}
 	}
 
-	depends := make([]string, 0, len(dependsCtrs))
-	for ctr := range dependsCtrs {
-		depends = append(depends, ctr)
-	}
-
-	return depends
+	return slices.Collect(maps.Keys(dependsCtrs))
 }
 
 // NewNetNS returns whether the container will create a new network namespace
@@ -622,9 +625,7 @@ func (c *Container) Stdin() bool {
 // Labels returns the container's labels
 func (c *Container) Labels() map[string]string {
 	labels := make(map[string]string)
-	for key, value := range c.config.Labels {
-		labels[key] = value
-	}
+	maps.Copy(labels, c.config.Labels)
 	return labels
 }
 
@@ -664,6 +665,14 @@ func (c *Container) LogTag() string {
 	return c.config.LogTag
 }
 
+// LogSizeMax returns the maximum size of the container's log file.
+func (c *Container) LogSizeMax() int64 {
+	if c.config.LogSize > 0 {
+		return c.config.LogSize
+	}
+	return c.runtime.config.Containers.LogSizeMax
+}
+
 // RestartPolicy returns the container's restart policy.
 func (c *Container) RestartPolicy() string {
 	return c.config.RestartPolicy
@@ -728,14 +737,14 @@ func (c *Container) hostname(network bool) string {
 	// containers.conf, use a sanitized version of the container's name
 	// as the hostname.  Since the container name must already match
 	// the set '[a-zA-Z0-9][a-zA-Z0-9_.-]*', we can just remove any
-	// underscores and limit it to 253 characters to make it a valid
+	// underscores and limit it to 64 characters to make it a valid
 	// hostname.
 	if c.runtime.config.Containers.ContainerNameAsHostName {
 		sanitizedHostname := strings.ReplaceAll(c.Name(), "_", "")
-		if len(sanitizedHostname) <= 253 {
+		if len(sanitizedHostname) <= 64 {
 			return sanitizedHostname
 		}
-		return sanitizedHostname[:253]
+		return sanitizedHostname[:64]
 	}
 
 	// Otherwise use the container's short ID as the hostname.
@@ -1027,9 +1036,7 @@ func (c *Container) BindMounts() (map[string]string, error) {
 
 	newMap := make(map[string]string, len(c.state.BindMounts))
 
-	for key, val := range c.state.BindMounts {
-		newMap[key] = val
-	}
+	maps.Copy(newMap, c.state.BindMounts)
 
 	return newMap, nil
 }
@@ -1154,7 +1161,7 @@ func (c *Container) cGroupPath() (string, error) {
 	}
 
 	var cgroupPath string
-	for _, line := range bytes.Split(lines, []byte("\n")) {
+	for line := range bytes.SplitSeq(lines, []byte("\n")) {
 		// skip last empty line
 		if len(line) == 0 {
 			continue
@@ -1310,7 +1317,20 @@ func (c *Container) HostNetwork() bool {
 // HasHealthCheck returns bool as to whether there is a health check
 // defined for the container
 func (c *Container) HasHealthCheck() bool {
-	return c.config.HealthCheckConfig != nil
+	// Consider a healthcheck present only when a HealthCheckConfig exists
+	// and the Test field contains a meaningful command. Treat an empty
+	// Test slice or the special ["NONE"] sentinel as "no healthcheck".
+	if c.config.HealthCheckConfig == nil {
+		return false
+	}
+	test := c.config.HealthCheckConfig.Test
+	if len(test) == 0 {
+		return false
+	}
+	if len(test) == 1 && strings.ToUpper(test[0]) == define.HealthConfigTestNone {
+		return false
+	}
+	return true
 }
 
 // HealthCheckConfig returns the command and timing attributes of the health check
