@@ -12,12 +12,13 @@ import (
 	"strconv"
 	"time"
 
-	"github.com/containers/common/pkg/resize"
-	"github.com/containers/common/pkg/util"
 	"github.com/containers/podman/v5/libpod/define"
 	"github.com/containers/podman/v5/libpod/events"
-	"github.com/containers/storage/pkg/stringid"
+	"github.com/containers/podman/v5/pkg/pidhandle"
 	"github.com/sirupsen/logrus"
+	"go.podman.io/common/pkg/resize"
+	"go.podman.io/common/pkg/util"
+	"go.podman.io/storage/pkg/stringid"
 	"golang.org/x/sys/unix"
 )
 
@@ -101,6 +102,10 @@ type ExecSession struct {
 	// Config is the configuration of this exec session.
 	// Cannot be empty.
 	Config *ExecConfig `json:"config"`
+
+	// PIDData is a string uniquely identifying the PID. The value is platform
+	// specific. It is generated by pidhandle package and used by isSessionAlive.
+	PIDData string `json:"pidData,omitempty"`
 }
 
 // ID returns the ID of an exec session.
@@ -175,14 +180,7 @@ func (c *Container) getUniqueExecSessionID() string {
 	found := true
 	// This really ought to be a do-while, but Go doesn't have those...
 	for found {
-		found = false
-		for id := range c.state.ExecSessions {
-			if id == sessionID {
-				found = true
-				break
-			}
-		}
-		if found {
+		if _, found = c.state.ExecSessions[sessionID]; found {
 			sessionID = stringid.GenerateRandomID()
 		}
 	}
@@ -251,6 +249,27 @@ func (c *Container) ExecCreate(config *ExecConfig) (string, error) {
 	return session.Id, nil
 }
 
+// Returns a serialized representation of the pid using the PIDHandle package.
+// This string can be passed to NewPIDHandleFromString to recreate a PIDHandle
+// that reliably refers to the same process as the original.
+//
+// NOTE that there is a still race in generating the PIDData on start but this
+// is as good as we can do currently. Long term we could change conmon to
+// send pidfd back directly.
+func getPidData(pid int) string {
+	pidHandle, err := pidhandle.NewPIDHandle(pid)
+	if err != nil {
+		logrus.Debugf("getting the PID handle for pid %d: %v", pid, err)
+		return ""
+	}
+	defer pidHandle.Close()
+	pidData, err := pidHandle.String()
+	if err != nil {
+		logrus.Debugf("generating PIDData (%d) failed: %v", pid, err)
+	}
+	return pidData
+}
+
 // ExecStart starts an exec session in the container, but does not attach to it.
 // Returns immediately upon starting the exec session, unlike other ExecStart
 // functions, which will only return when the exec session exits.
@@ -296,6 +315,7 @@ func (c *Container) ExecStart(sessionID string) error {
 	// Update and save session to reflect PID/running
 	session.PID = pid
 	session.State = define.ExecStateRunning
+	session.PIDData = getPidData(pid)
 
 	return c.save()
 }
@@ -354,6 +374,7 @@ func (c *Container) execStartAndAttach(sessionID string, streams *define.AttachS
 	// Update and save session to reflect PID/running
 	session.PID = pid
 	session.State = define.ExecStateRunning
+	session.PIDData = getPidData(pid)
 
 	if err := c.save(); err != nil {
 		lastErr = err
@@ -450,7 +471,7 @@ func (c *Container) execStartAndAttach(sessionID string, streams *define.AttachS
 // ExecHTTPStartAndAttach starts and performs an HTTP attach to an exec session.
 // newSize resizes the tty to this size before the process is started, must be nil if the exec session has no tty
 func (c *Container) ExecHTTPStartAndAttach(sessionID string, r *http.Request, w http.ResponseWriter,
-	streams *HTTPAttachStreams, detachKeys *string, cancel <-chan bool, hijackDone chan<- bool, newSize *resize.TerminalSize) error {
+	streams *HTTPAttachStreams, _ *string, cancel <-chan bool, hijackDone chan<- bool, newSize *resize.TerminalSize) error {
 	// TODO: How do we combine streams with the default streams set in the exec session?
 
 	// Ensure that we don't leak a goroutine here
@@ -535,6 +556,7 @@ func (c *Container) ExecHTTPStartAndAttach(sessionID string, r *http.Request, w
 
 	session.PID = pid
 	session.State = define.ExecStateRunning
+	session.PIDData = getPidData(pid)
 
 	if err := c.save(); err != nil {
 		lastErr = err
@@ -791,66 +813,8 @@ func (c *Container) ExecResize(sessionID string, newSize resize.TerminalSize) er
 	return c.ociRuntime.ExecAttachResize(c, sessionID, newSize)
 }
 
-func (c *Container) healthCheckExec(config *ExecConfig, streams *define.AttachStreams) (int, error) {
-	unlock := true
-	if !c.batched {
-		c.lock.Lock()
-		defer func() {
-			if unlock {
-				c.lock.Unlock()
-			}
-		}()
-
-		if err := c.syncContainer(); err != nil {
-			return -1, err
-		}
-	}
-
-	if err := c.verifyExecConfig(config); err != nil {
-		return -1, err
-	}
-
-	if !c.ensureState(define.ContainerStateRunning) {
-		return -1, fmt.Errorf("can only create exec sessions on running containers: %w", define.ErrCtrStateInvalid)
-	}
-
-	session, err := c.createExecSession(config)
-	if err != nil {
-		return -1, err
-	}
-
-	if c.state.ExecSessions == nil {
-		c.state.ExecSessions = make(map[string]*ExecSession)
-	}
-	c.state.ExecSessions[session.ID()] = session
-	defer delete(c.state.ExecSessions, session.ID())
-
-	opts, err := prepareForExec(c, session)
-	if err != nil {
-		return -1, err
-	}
-	defer func() {
-		if err := c.cleanupExecBundle(session.ID()); err != nil {
-			logrus.Errorf("Container %s light exec session cleanup error: %v", c.ID(), err)
-		}
-	}()
-
-	pid, attachErrChan, err := c.ociRuntime.ExecContainer(c, session.ID(), opts, streams, nil)
-	if err != nil {
-		return -1, err
-	}
-
-	if !c.batched {
-		c.lock.Unlock()
-		unlock = false
-	}
-
-	err = <-attachErrChan
-	if err != nil {
-		return -1, fmt.Errorf("container %s light exec session with pid: %d error: %v", c.ID(), pid, err)
-	}
-
-	return c.readExecExitCode(session.ID())
+func (c *Container) healthCheckExec(config *ExecConfig, timeout time.Duration, streams *define.AttachStreams) (int, error) {
+	return c.execLightweight(config, streams, timeout)
 }
 
 func (c *Container) Exec(config *ExecConfig, streams *define.AttachStreams, resize <-chan resize.TerminalSize) (int, error) {
@@ -933,13 +897,14 @@ func (c *Container) exec(config *ExecConfig, streams *define.AttachStreams, resi
 	return session.ExitCode, nil
 }
 
-// cleanupExecBundle cleanups an exec session after its done
+// cleanupExecBundle cleans up an exec session after completion.
+// MUST BE CALLED with container `c` locked.
 // Please be careful when using this function since it might temporarily unlock
 // the container when os.RemoveAll($bundlePath) fails with ENOTEMPTY or EBUSY
 // errors.
 func (c *Container) cleanupExecBundle(sessionID string) (err error) {
 	path := c.execBundlePath(sessionID)
-	for attempts := 0; attempts < 50; attempts++ {
+	for range 50 {
 		err = os.RemoveAll(path)
 		if err == nil || os.IsNotExist(err) {
 			return nil
@@ -1057,17 +1022,17 @@ func (c *Container) readExecExitCode(sessionID string) (int, error) {
 }
 
 // getExecSessionPID gets the PID of an active exec session
-func (c *Container) getExecSessionPID(sessionID string) (int, error) {
+func (c *Container) getExecSessionPID(sessionID string) (int, string, error) {
 	session, ok := c.state.ExecSessions[sessionID]
 	if ok {
-		return session.PID, nil
+		return session.PID, session.PIDData, nil
 	}
 	oldSession, ok := c.state.LegacyExecSessions[sessionID]
 	if ok {
-		return oldSession.PID, nil
+		return oldSession.PID, "", nil
 	}
 
-	return -1, fmt.Errorf("no exec session with ID %s found in container %s: %w", sessionID, c.ID(), define.ErrNoSuchExecSession)
+	return -1, "", fmt.Errorf("no exec session with ID %s found in container %s: %w", sessionID, c.ID(), define.ErrNoSuchExecSession)
 }
 
 // getKnownExecSessions gets a list of all exec sessions we think are running,
@@ -1128,6 +1093,7 @@ func (c *Container) getActiveExecSessions() ([]string, error) {
 				}
 				session.ExitCode = exitCode
 				session.PID = 0
+				session.PIDData = ""
 				session.State = define.ExecStateStopped
 
 				c.newExecDiedEvent(session.ID(), exitCode)
@@ -1262,7 +1228,104 @@ func justWriteExecExitCode(c *Container, sessionID string, exitCode int, emitEve
 	session.State = define.ExecStateStopped
 	session.ExitCode = exitCode
 	session.PID = 0
+	session.PIDData = ""
 
 	// Finally, save our changes.
 	return c.save()
 }
+
+// execLightweight executes a command in a container without creating a persistent exec session.
+// It is used by both ExecNoSession and healthCheckExec to avoid code duplication.
+func (c *Container) execLightweight(config *ExecConfig, streams *define.AttachStreams, timeout time.Duration) (int, error) {
+	if err := c.verifyExecConfig(config); err != nil {
+		return -1, err
+	}
+
+	unlock := true
+	if !c.batched {
+		c.lock.Lock()
+		defer func() {
+			if unlock {
+				c.lock.Unlock()
+			}
+		}()
+
+		if err := c.syncContainer(); err != nil {
+			return -1, err
+		}
+	}
+
+	if !c.ensureState(define.ContainerStateRunning) {
+		return -1, fmt.Errorf("can only create exec sessions on running containers: %w", define.ErrCtrStateInvalid)
+	}
+
+	session, err := c.createExecSession(config)
+	if err != nil {
+		return -1, err
+	}
+
+	if c.state.ExecSessions == nil {
+		c.state.ExecSessions = make(map[string]*ExecSession)
+	}
+	c.state.ExecSessions[session.ID()] = session
+	defer delete(c.state.ExecSessions, session.ID())
+
+	opts, err := prepareForExec(c, session)
+	if err != nil {
+		return -1, err
+	}
+	defer func() {
+		if err := c.cleanupExecBundle(session.ID()); err != nil {
+			logrus.Errorf("Container %s light exec session cleanup error: %v", c.ID(), err)
+		}
+	}()
+
+	pid, attachErrChan, err := c.ociRuntime.ExecContainer(c, session.ID(), opts, streams, nil)
+	if err != nil {
+		// Check if the error is command not found before returning
+		if exitCode := define.ExitCode(err); exitCode == define.ExecErrorCodeNotFound {
+			return exitCode, err
+		}
+		return define.ExecErrorCodeGeneric, err
+	}
+	session.PID = pid
+	session.PIDData = getPidData(pid)
+
+	if !c.batched {
+		c.lock.Unlock()
+		unlock = false
+	}
+
+	// Handle timeout for health checks
+	if timeout > 0 {
+		select {
+		case err = <-attachErrChan:
+			if err != nil {
+				return -1, fmt.Errorf("container %s light exec session with pid: %d error: %v", c.ID(), pid, err)
+			}
+		case <-time.After(timeout):
+			if err := c.ociRuntime.ExecStopContainer(c, session.ID(), 0); err != nil {
+				return -1, err
+			}
+			return -1, fmt.Errorf("%v of %s", define.ErrHealthCheckTimeout, timeout.String())
+		}
+	} else {
+		// For no-session exec, wait for completion without timeout
+		err = <-attachErrChan
+		if err != nil && !errors.Is(err, define.ErrDetach) {
+			// Check if the error is command not found
+			if exitCode := define.ExitCode(err); exitCode == define.ExecErrorCodeNotFound {
+				return exitCode, err
+			}
+			return define.ExecErrorCodeGeneric, err
+		}
+	}
+
+	return c.readExecExitCode(session.ID())
+}
+
+// ExecNoSession executes a command in a container without creating a persistent exec session.
+// It skips database operations and minimizes container locking for performance.
+func (c *Container) ExecNoSession(config *ExecConfig, streams *define.AttachStreams, _ <-chan resize.TerminalSize) (int, error) {
+	return c.execLightweight(config, streams, 0)
+}
