@@ -5,6 +5,7 @@ package libpod
 import (
 	"errors"
 	"fmt"
+	"maps"
 	"strings"
 
 	"github.com/containers/podman/v5/libpod/define"
@@ -191,10 +192,7 @@ func (c *Container) getContainerInspectData(size bool, driverData *define.Driver
 		data.OCIConfigPath = c.state.ConfigPath
 	}
 
-	// Check if healthcheck is not nil and --no-healthcheck option is not set.
-	// If --no-healthcheck is set Test will be always set to `[NONE]`, so the
-	// inspect status should be set to nil.
-	if c.config.HealthCheckConfig != nil && (len(c.config.HealthCheckConfig.Test) != 1 || c.config.HealthCheckConfig.Test[0] != "NONE") {
+	if c.HasHealthCheck() {
 		// This container has a healthcheck defined in it; we need to add its state
 		healthCheckState, err := c.readHealthCheckLog()
 		if err != nil {
@@ -415,17 +413,11 @@ func (c *Container) generateInspectContainerConfig(spec *spec.Spec) *define.Insp
 	}
 
 	if len(c.config.Labels) != 0 {
-		ctrConfig.Labels = make(map[string]string)
-		for k, v := range c.config.Labels {
-			ctrConfig.Labels[k] = v
-		}
+		ctrConfig.Labels = maps.Clone(c.config.Labels)
 	}
 
 	if len(spec.Annotations) != 0 {
-		ctrConfig.Annotations = make(map[string]string)
-		for k, v := range spec.Annotations {
-			ctrConfig.Annotations[k] = v
-		}
+		ctrConfig.Annotations = maps.Clone(spec.Annotations)
 	}
 	ctrConfig.StopSignal = signal.ToDockerFormat(c.config.StopSignal)
 	// TODO: should JSON deep copy this to ensure internal pointers don't
@@ -498,7 +490,7 @@ func (c *Container) generateInspectContainerHostConfig(ctrSpec *spec.Spec, named
 	logConfig := new(define.InspectLogConfig)
 	logConfig.Type = c.config.LogDriver
 	logConfig.Path = c.config.LogPath
-	logConfig.Size = units.HumanSize(float64(c.config.LogSize))
+	logConfig.Size = units.HumanSize(float64(c.LogSizeMax()))
 	logConfig.Tag = c.config.LogTag
 
 	hostConfig.LogConfig = logConfig
