@@ -658,6 +658,10 @@ func (ic *ContainerEngine) ContainerExec(_ context.Context, nameOrID string, opt
 	return inspectOut.ExitCode, nil
 }
 
+func (ic *ContainerEngine) ContainerExecNoSession(_ context.Context, _ string, _ entities.ExecOptions, _ define.AttachStreams) (int, error) {
+	return 0, errors.New("--no-session is not supported for the remote client")
+}
+
 func (ic *ContainerEngine) ContainerExecDetached(_ context.Context, nameOrID string, options entities.ExecOptions) (retSessionID string, retErr error) {
 	createConfig := makeExecConfig(options)
 
