@@ -13,10 +13,8 @@ import (
 	"github.com/containers/podman/v5/libpod/define"
 	"github.com/containers/podman/v5/pkg/rootless"
 	"github.com/containers/podman/v5/pkg/specgen"
-	"github.com/docker/go-units"
 	spec "github.com/opencontainers/runtime-spec/specs-go"
 	"github.com/opencontainers/runtime-tools/generate"
-	"github.com/sirupsen/logrus"
 	"go.podman.io/common/libimage"
 	"go.podman.io/common/pkg/cgroups"
 	"go.podman.io/common/pkg/config"
@@ -369,37 +367,3 @@ func WeightDevices(wtDevices map[string]spec.LinuxWeightDevice) ([]spec.LinuxWei
 	}
 	return devs, nil
 }
-
-// subNegativeOne translates Hard or soft limits of -1 to the current
-// processes Max limit
-func subNegativeOne(u spec.POSIXRlimit) spec.POSIXRlimit {
-	if !rootless.IsRootless() ||
-		(int64(u.Hard) != -1 && int64(u.Soft) != -1) {
-		return u
-	}
-
-	ul, err := units.ParseUlimit(fmt.Sprintf("%s=%d:%d", u.Type, int64(u.Soft), int64(u.Hard)))
-	if err != nil {
-		logrus.Warnf("Failed to check %s ulimit %q", u.Type, err)
-		return u
-	}
-	rl, err := ul.GetRlimit()
-	if err != nil {
-		logrus.Warnf("Failed to check %s ulimit %q", u.Type, err)
-		return u
-	}
-
-	var rlimit unix.Rlimit
-
-	if err := unix.Getrlimit(rl.Type, &rlimit); err != nil {
-		logrus.Warnf("Failed to return RLIMIT_NOFILE ulimit %q", err)
-		return u
-	}
-	if int64(u.Hard) == -1 {
-		u.Hard = rlimit.Max
-	}
-	if int64(u.Soft) == -1 {
-		u.Soft = rlimit.Max
-	}
-	return u
-}
